@@ -38,12 +38,13 @@
 #include <string.h>
 #include <time.h>
 
-#include <speex/speex.h>
+#include "speex.h"
 #include <ogg/ogg.h>
 #include "wav_io.h"
-#include <speex/speex_header.h>
-#include <speex/speex_stereo.h>
+#include "speex_header.h"
+#include "speex_stereo.h"
 #include "misc.h"
+#include "speex_preprocess.h"
 
 #if defined WIN32 || defined _WIN32
 #include "getopt_win.h"
@@ -80,7 +81,7 @@ static int read_samples(FILE *fin,int frame_size, int bits, int channels, int ls
 
    if (size && *size<=0)
    {
-      return 0;
+      return 1;
    }
    /*Read input audio*/
    if (size)
@@ -99,7 +100,7 @@ static int read_samples(FILE *fin,int frame_size, int bits, int channels, int ls
 
    /*fprintf (stderr, "%d\n", nb_read);*/
    if (nb_read==0)
-      return 0;
+      return 1;
 
    s=(short*)in;
    if(bits==8)
@@ -134,18 +135,18 @@ static int read_samples(FILE *fin,int frame_size, int bits, int channels, int ls
    }
 
 
-   return nb_read;
+   return 0;
 }
 
 void version()
 {
-   printf ("speexenc (Speex encoder) version " SPEEX_VERSION " (compiled " __DATE__ ")\n");
+   printf ("speexenc (Speex encoder) version " VERSION " (compiled " __DATE__ ")\n");
    printf ("Copyright (C) 2002-2003 Jean-Marc Valin\n");
 }
 
 void version_short()
 {
-   printf ("speexenc version " SPEEX_VERSION "\n");
+   printf ("speexenc version " VERSION "\n");
    printf ("Copyright (C) 2002-2003 Jean-Marc Valin\n");
 }
 
@@ -176,6 +177,8 @@ void usage()
    printf (" --dtx              Enable file-based discontinuous transmission (DTX)\n"); 
    printf (" --comp n           Set encoding complexity (0-10), default 3\n"); 
    printf (" --nframes n        Number of frames per Ogg packet (1-10), default 1\n"); 
+   printf (" --denoise          Denoise the input before encoding\n"); 
+   printf (" --agc              Apply adaptive gain control (AGC) before encoding\n"); 
    printf (" --comment          Add the given string as an extra comment. This may be\n");
    printf ("                     used multiple times\n");
    printf (" --author           Author of this track\n");
@@ -200,7 +203,6 @@ void usage()
 
 int main(int argc, char **argv)
 {
-   int nb_samples, total_samples=0, nb_encoded;
    int c;
    int option_index = 0;
    char *inFile, *outFile;
@@ -213,8 +215,7 @@ int main(int argc, char **argv)
    int vad_enabled=0;
    int dtx_enabled=0;
    int nbBytes;
-   SpeexMode *mode=NULL;
-   int modeID = -1;
+   const SpeexMode *mode=NULL;
    void *st;
    SpeexBits bits;
    char cbits[MAX_FRAME_BYTES];
@@ -231,6 +232,8 @@ int main(int argc, char **argv)
       {"bitrate", required_argument, NULL, 0},
       {"nframes", required_argument, NULL, 0},
       {"comp", required_argument, NULL, 0},
+      {"denoise", no_argument, NULL, 0},
+      {"agc", no_argument, NULL, 0},
       {"help", no_argument, NULL, 0},
       {"quiet", no_argument, NULL, 0},
       {"le", no_argument, NULL, 0},
@@ -261,7 +264,7 @@ int main(int argc, char **argv)
    SpeexHeader header;
    int nframes=1;
    int complexity=3;
-   char *vendor_string = "Encoded with Speex " SPEEX_VERSION;
+   char *vendor_string = "Encoded with Speex " VERSION;
    char *comments;
    int comments_length;
    int close_in=0, close_out=0;
@@ -271,8 +274,9 @@ int main(int argc, char **argv)
    char first_bytes[12];
    int wave_input=0;
    int tmp;
-   int lookahead = 0;
-   
+   SpeexPreprocessState *preprocess = NULL;
+   int denoise_enabled=0, agc_enabled=0;
+
    comment_init(&comments, &comments_length, vendor_string);
 
    /*Process command-line options*/
@@ -288,13 +292,13 @@ int main(int argc, char **argv)
       case 0:
          if (strcmp(long_options[option_index].name,"narrowband")==0)
          {
-            modeID = SPEEX_MODEID_NB;
+            mode=&speex_nb_mode;
          } else if (strcmp(long_options[option_index].name,"wideband")==0)
          {
-            modeID = SPEEX_MODEID_WB;
+            mode=&speex_wb_mode;
          } else if (strcmp(long_options[option_index].name,"ultra-wideband")==0)
          {
-            modeID = SPEEX_MODEID_UWB;
+            mode=&speex_uwb_mode;
          } else if (strcmp(long_options[option_index].name,"vbr")==0)
          {
             vbr_enabled=1;
@@ -329,6 +333,12 @@ int main(int argc, char **argv)
          } else if (strcmp(long_options[option_index].name,"comp")==0)
          {
             complexity = atoi (optarg);
+         } else if (strcmp(long_options[option_index].name,"denoise")==0)
+         {
+            denoise_enabled=1;
+         } else if (strcmp(long_options[option_index].name,"agc")==0)
+         {
+            agc_enabled=1;
          } else if (strcmp(long_options[option_index].name,"help")==0)
          {
             usage();
@@ -364,12 +374,6 @@ int main(int argc, char **argv)
             rate=atoi (optarg);
          } else if (strcmp(long_options[option_index].name,"comment")==0)
          {
-	   if (!strchr(optarg, '='))
-	   {
-	     fprintf (stderr, "Invalid comment: %s\n", optarg);
-	     fprintf (stderr, "Comments must be of the form name=value\n");
-	     exit(1);
-	   }
            comment_add(&comments, &comments_length, NULL, optarg); 
          } else if (strcmp(long_options[option_index].name,"author")==0)
          {
@@ -381,7 +385,7 @@ int main(int argc, char **argv)
 
          break;
       case 'n':
-         modeID = SPEEX_MODEID_NB;
+         mode=&speex_nb_mode;
          break;
       case 'h':
          usage();
@@ -395,10 +399,10 @@ int main(int argc, char **argv)
          print_bitrate=1;
          break;
       case 'w':
-         modeID = SPEEX_MODEID_WB;
+         mode=&speex_wb_mode;
          break;
       case 'u':
-         modeID = SPEEX_MODEID_UWB;
+         mode=&speex_uwb_mode;
          break;
       case '?':
          usage();
@@ -431,7 +435,11 @@ int main(int argc, char **argv)
    }
    else 
    {
+#if defined WIN32 || defined _WIN32
       fin = fopen(inFile, "rb");
+#else
+      fin = fopen(inFile, "r");
+#endif
       if (!fin)
       {
          perror(inFile);
@@ -451,12 +459,12 @@ int main(int argc, char **argv)
       }
    }
 
-   if (modeID==-1 && !rate)
+   if (!mode && !rate)
    {
       /* By default, use narrowband/8 kHz */
-      modeID = SPEEX_MODEID_NB;
+      mode=&speex_nb_mode;
       rate=8000;
-   } else if (modeID!=-1 && rate)
+   } else if (mode && rate)
    {
       if (rate>48000)
       {
@@ -464,19 +472,19 @@ int main(int argc, char **argv)
          exit(1);
       } else if (rate>25000)
       {
-         if (modeID != SPEEX_MODEID_UWB)
+         if (mode!=&speex_uwb_mode)
          {
             fprintf (stderr, "Warning: Trying to encode in %s at %d Hz. I'll do it but I suggest you try ultra-wideband instead\n", mode->modeName , rate);
          }
       } else if (rate>12500)
       {
-         if (modeID != SPEEX_MODEID_WB)
+         if (mode!=&speex_wb_mode)
          {
             fprintf (stderr, "Warning: Trying to encode in %s at %d Hz. I'll do it but I suggest you try wideband instead\n", mode->modeName , rate);
          }
       } else if (rate>=6000)
       {
-         if (modeID != SPEEX_MODEID_NB)
+         if (mode!=&speex_nb_mode)
          {
             fprintf (stderr, "Warning: Trying to encode in %s at %d Hz. I'll do it but I suggest you try narrowband instead\n", mode->modeName , rate);
          }
@@ -484,7 +492,7 @@ int main(int argc, char **argv)
          fprintf (stderr, "Error: sampling rate too low: %d Hz\n", rate);
          exit(1);
       }
-   } else if (modeID==-1)
+   } else if (!mode)
    {
       if (rate>48000)
       {
@@ -492,24 +500,24 @@ int main(int argc, char **argv)
          exit(1);
       } else if (rate>25000)
       {
-         modeID = SPEEX_MODEID_UWB;
+         mode=&speex_uwb_mode;
       } else if (rate>12500)
       {
-         modeID = SPEEX_MODEID_WB;
+         mode=&speex_wb_mode;
       } else if (rate>=6000)
       {
-         modeID = SPEEX_MODEID_NB;
+         mode=&speex_nb_mode;
       } else {
          fprintf (stderr, "Error: Sampling rate too low: %d Hz\n", rate);
          exit(1);
       }
    } else if (!rate)
    {
-      if (modeID == SPEEX_MODEID_NB)
+      if (mode==&speex_nb_mode)
          rate=8000;
-      else if (modeID == SPEEX_MODEID_WB)
+      else if (mode==&speex_wb_mode)
          rate=16000;
-      else if (modeID == SPEEX_MODEID_UWB)
+      else if (mode==&speex_uwb_mode)
          rate=32000;
    }
 
@@ -517,8 +525,6 @@ int main(int argc, char **argv)
       if (rate!=8000 && rate!=16000 && rate!=32000)
          fprintf (stderr, "Warning: Speex is only optimized for 8, 16 and 32 kHz. It will still work at %d Hz but your mileage may vary\n", rate); 
 
-   mode = speex_lib_get_mode (modeID);
-
    speex_init_header(&header, rate, 1, mode);
    header.frames_per_packet=nframes;
    header.vbr=vbr_enabled;
@@ -547,7 +553,11 @@ int main(int argc, char **argv)
    }
    else 
    {
+#if defined WIN32 || defined _WIN32
       fout = fopen(outFile, "wb");
+#else
+      fout = fopen(outFile, "w");
+#endif
       if (!fout)
       {
          perror(outFile);
@@ -556,6 +566,42 @@ int main(int argc, char **argv)
       close_out=1;
    }
 
+
+   /*Write header (format will change)*/
+   {
+
+      op.packet = (unsigned char *)speex_header_to_packet(&header, (int*)&(op.bytes));
+      op.b_o_s = 1;
+      op.e_o_s = 0;
+      op.granulepos = 0;
+      op.packetno = 0;
+      ogg_stream_packetin(&os, &op);
+      free(op.packet);
+
+      op.packet = (unsigned char *)comments;
+      op.bytes = comments_length;
+      op.b_o_s = 0;
+      op.e_o_s = 0;
+      op.granulepos = 0;
+      op.packetno = 1;
+      ogg_stream_packetin(&os, &op);
+      
+      while((result = ogg_stream_flush(&os, &og)))
+      {
+         if(!result) break;
+         ret = oe_write_page(&og, fout);
+         if(ret != og.header_len + og.body_len)
+         {
+            fprintf (stderr,"Error: failed writing header to output stream\n");
+            exit(1);
+         }
+         else
+            bytes_written += ret;
+      }
+   }
+
+   free(comments);
+
    speex_encoder_ctl(st, SPEEX_GET_FRAME_SIZE, &frame_size);
    speex_encoder_ctl(st, SPEEX_SET_COMPLEXITY, &complexity);
    speex_encoder_ctl(st, SPEEX_SET_SAMPLING_RATE, &rate);
@@ -597,66 +643,36 @@ int main(int argc, char **argv)
       speex_encoder_ctl(st, SPEEX_SET_ABR, &abr_enabled);
    }
 
-   speex_encoder_ctl(st, SPEEX_GET_LOOKAHEAD, &lookahead);
-   
-   /*Write header*/
+   if (denoise_enabled || agc_enabled)
    {
-
-      op.packet = (unsigned char *)speex_header_to_packet(&header, (int*)&(op.bytes));
-      op.b_o_s = 1;
-      op.e_o_s = 0;
-      op.granulepos = 0;
-      op.packetno = 0;
-      ogg_stream_packetin(&os, &op);
-      free(op.packet);
-
-      op.packet = (unsigned char *)comments;
-      op.bytes = comments_length;
-      op.b_o_s = 0;
-      op.e_o_s = 0;
-      op.granulepos = 0;
-      op.packetno = 1;
-      ogg_stream_packetin(&os, &op);
-      
-      while((result = ogg_stream_flush(&os, &og)))
-      {
-         if(!result) break;
-         ret = oe_write_page(&og, fout);
-         if(ret != og.header_len + og.body_len)
-         {
-            fprintf (stderr,"Error: failed writing header to output stream\n");
-            exit(1);
-         }
-         else
-            bytes_written += ret;
-      }
+      preprocess = speex_preprocess_state_init(frame_size, rate);
+      speex_preprocess_ctl(preprocess, SPEEX_PREPROCESS_SET_DENOISE, &denoise_enabled);
+      speex_preprocess_ctl(preprocess, SPEEX_PREPROCESS_SET_AGC, &agc_enabled);
    }
 
-   free(comments);
-
    speex_bits_init(&bits);
 
    if (!wave_input)
    {
-      nb_samples = read_samples(fin,frame_size,fmt,chan,lsb,input, first_bytes, NULL);
+      if (read_samples(fin,frame_size,fmt,chan,lsb,input, first_bytes, NULL))
+         eos=1;
    } else {
-      nb_samples = read_samples(fin,frame_size,fmt,chan,lsb,input, NULL, &size);
+      if (read_samples(fin,frame_size,fmt,chan,lsb,input, NULL, &size))
+         eos=1;
    }
-   if (nb_samples==0)
-      eos=1;
-   total_samples += nb_samples;
-   nb_encoded = -lookahead;
    /*Main encoding loop (one frame per iteration)*/
-   while (!eos || total_samples>nb_encoded)
+   while (!eos)
    {
       id++;
       /*Encode current frame*/
       if (chan==2)
-         speex_encode_stereo_int(input, frame_size, &bits);
+         speex_encode_stereo(input, frame_size, &bits);
 
-      speex_encode_int(st, input, &bits);
+      if (preprocess)
+         speex_preprocess(preprocess, input, NULL);
+
+      speex_encode(st, input, &bits);
       
-      nb_encoded += frame_size;
       if (print_bitrate) {
          int tmp;
          char ch=13;
@@ -676,19 +692,18 @@ int main(int argc, char **argv)
 
       if (wave_input)
       {
-         nb_samples = read_samples(fin,frame_size,fmt,chan,lsb,input, NULL, &size);
+         if (read_samples(fin,frame_size,fmt,chan,lsb,input, NULL, &size))
+         {
+            eos=1;
+            op.e_o_s = 1;
+         }
       } else {
-         nb_samples = read_samples(fin,frame_size,fmt,chan,lsb,input, NULL, NULL);
+         if (read_samples(fin,frame_size,fmt,chan,lsb,input, NULL, NULL))
+         {
+            eos=1;
+            op.e_o_s = 1;
+         }
       }
-      if (nb_samples==0)
-      {
-         eos=1;
-      }
-      if (eos && total_samples<=nb_encoded)
-         op.e_o_s = 1;
-      else
-         op.e_o_s = 0;
-      total_samples += nb_samples;
 
       if ((id+1)%nframes!=0)
          continue;
@@ -698,15 +713,11 @@ int main(int argc, char **argv)
       op.packet = (unsigned char *)cbits;
       op.bytes = nbBytes;
       op.b_o_s = 0;
-      /*Is this redundent?*/
-      if (eos && total_samples<=nb_encoded)
+      if (eos)
          op.e_o_s = 1;
       else
          op.e_o_s = 0;
-      op.granulepos = (id+1)*frame_size-lookahead;
-      if (op.granulepos>total_samples)
-         op.granulepos = total_samples;
-      /*printf ("granulepos: %d %d %d %d %d %d\n", (int)op.granulepos, id, nframes, lookahead, 5, 6);*/
+      op.granulepos = (id+nframes)*frame_size;
       op.packetno = 2+id/nframes;
       ogg_stream_packetin(&os, &op);
 
@@ -735,10 +746,7 @@ int main(int argc, char **argv)
       op.bytes = nbBytes;
       op.b_o_s = 0;
       op.e_o_s = 1;
-      op.granulepos = (id+1)*frame_size-lookahead;
-      if (op.granulepos>total_samples)
-         op.granulepos = total_samples;
-
+      op.granulepos = (id+nframes)*frame_size;
       op.packetno = 2+id/nframes;
       ogg_stream_packetin(&os, &op);
    }
@@ -754,6 +762,7 @@ int main(int argc, char **argv)
       else
          bytes_written += ret;
    }
+   
 
    speex_encoder_destroy(st);
    speex_bits_destroy(&bits);
